@@ -18,13 +18,13 @@
 #
 
 import sys, os, fcntl, pwd, grp, signal, errno, time, socket, select, yaml, re
-import utils
-from utils import ses, deltafmt, statusfmt
-from utils import get_caller as my
-import poll
-import watch_files
-import watch_modules
 import logging
+from . import utils
+from .utils import ses, deltafmt, statusfmt
+from .utils import get_caller as my
+from . import poll
+from . import watch_files
+from . import watch_modules
 
 #  The seconds before a SIGTERM sent to a task is
 #  escalated to a SIGKILL.
@@ -654,7 +654,7 @@ Params are:
 		if sig == signal.SIGCHLD:
 			log.debug("%s Received SIGCHLD", my(self))
 			try:
-				os.write(self._wakeup, '*')
+				os.write(self._wakeup, '*'.encode('utf-8'))
 			except Exception as e:
 				log.error("%s Write to self-pipe failed -- %s", my(self), str(e))
 			return
@@ -1130,7 +1130,9 @@ Params are:
 							self._load_config()
 							self._reload_config = None
 						except Exception as e:
-							log.error("%s Config load sequence failed -- %s", my(self), str(e))
+
+							log.error("%s Config load sequence failed -- %s",
+											my(self), str(e), exc_info=True)
 
 					#  Housekeeping
 					#
@@ -1267,7 +1269,7 @@ Params are:
 		#  Register with legion
 		self._legion.task_add(self, periodic=self._task_periodic)
 
-	def __del__(self):
+	def close(self):
 		if self._legion:
 			try: self._event_deregister()
 			except: pass
@@ -1649,7 +1651,7 @@ Params are:
 		if self._config_running is None:
 			log.debug("%s Task '%s' change - no previous config", my(self), self._name)
 			return True
-		for elem in list(set(self._config_running.keys() + self._config_pending.keys())):
+		for elem in set(list(self._config_running) + list(self._config_pending)):
 			#  Ignore these elements as they don't affect the operation of a process
 			#  that is already running
 			#
@@ -1662,7 +1664,7 @@ Params are:
 		if self._context != new_context:
 			if log.isEnabledFor(logging.DEBUG):
 				log.debug("%s Task '%s' change - context change", my(self), self._name)
-				for tag in list(set(self._context.keys() + new_context.keys())):
+				for tag in set(list(self._context) + list(new_context)):
 					o = self._context.get(tag)
 					n = new_context.get(tag)
 					if o != n:
@@ -1792,7 +1794,7 @@ Params are:
 		if self._stopped:
 			if self._dnr:
 				log.info("%s Task %s stopped and will now be deleted", my(self), self._name)
-				self.__del__()
+				self.close()
 				return False
 			elif once:
 				log.debug("%s 'once' task %s exited %s ago",
@@ -2045,46 +2047,3 @@ Params are:
 			return False
 		log.debug("%s managing '%s'", my(self), self._name)
 		return self._start()
-
-if __name__ == "__main__":
-	import argparse
-
-	def_roles_file = 'tf_roles.conf'
-	def_config_file = 'taskforce.conf'
-
-	p = argparse.ArgumentParser(
-		formatter_class=argparse.RawDescriptionHelpFormatter,
-		description=utils.module_description(__name__, __doc__, __file__))
-
-	p.add_argument('-v', '--verbose', action='store_true', dest='verbose', help='Verbose logging for debugging')
-	p.add_argument('-q', '--quiet', action='store_true', dest='quiet', help='Warnings and errors only')
-	p.add_argument('-f', '--config-file', action='store', dest='config_file', default=def_config_file,
-			help='Configuration.  File will be watched for changes.  Default '+def_config_file)
-	p.add_argument('-r', '--roles-file', action='store', dest='roles_file', default=def_roles_file,
-			help='File to load roles from.  File will be watched for changes.  Default '+def_roles_file)
-	p.add_argument('-C', '--check-config', action='store_true', dest='check', help='Check the config and exit')
-
-	args = p.parse_args()
-
-	log = logging.getLogger()
-	log.addHandler(logging.StreamHandler())
-
-	if args.verbose:
-		log.setLevel(logging.DEBUG)
-	elif args.quiet:
-		log.setLevel(logging.WARNING)
-	else:
-		log.setLevel(logging.INFO)
-
-	try:
-		l = legion(log=log)
-		l.set_roles_file(args.roles_file)
-		l.set_config_file(args.config_file)
-		if args.check:
-			log.info("Config file '%s' appears valid", args.config_file)
-			sys.exit(0)
-		l.manage()
-	except Exception as e:
-		log.error('Error -- %s', str(e), exc_info=args.verbose)
-		sys.exit(2)
-	sys.exit(0)
