@@ -18,9 +18,9 @@
 #
 
 import sys, os, time, errno, select, logging
-import utils
-from utils import ses
-from utils import get_caller as my
+from . import utils
+from .utils import ses
+from .utils import get_caller as my
 
 #  These values are used internally to select watch mode.
 #
@@ -175,6 +175,9 @@ class watch(object):
 			self._poll_pending = {}
 
 	def __del__(self):
+		self.close()
+
+	def close(self):
 		close_fds = True
 		if self._mode == WF_KQUEUE and self._kq:
 			#  The is actually auto-closed, so this bit is not
@@ -201,7 +204,7 @@ class watch(object):
 		#  definitely need the destructor here.
 		#
 		if close_fds:
-			for fd in self.fds_open.keys():
+			for fd in list(self.fds_open):
 				try: os.close(fd)
 				except: pass
 				del self.fds_open[fd]
@@ -307,7 +310,7 @@ class watch(object):
 		Trigger activity for the caller by writting a NUL to the self-pipe.
 	"""
 		try:
-			os.write(self._poll_send, '\0')
+			os.write(self._poll_send, '\0'.encode('utf-8'))
 		except Exception as e:
 			log = self._getparam('log', self._discard)
 			log.debug("%s Ignoring self-pipe write error -- %s", my(self), str(e))
@@ -442,7 +445,7 @@ class watch(object):
 		#
 		removed = 0
 		added = 0
-		for path in self.paths_open.keys():
+		for path in list(self.paths_open):
 			if path not in self.paths:
 				fd = self.paths_open[path]
 				if self._mode == WF_KQUEUE:
@@ -461,6 +464,10 @@ class watch(object):
 						del self._poll_stat[fd]
 					else:
 						log.warning("%s fd watched path '%s' missing from _poll_stat map", my(self), path)
+					try:
+						os.close(fd)
+					except Exception as e:
+						log.warning("%s close failed on watched file '%s' -- %s", my(self), path, str(e))
 				if fd in self.fds_open:
 					del self.fds_open[fd]
 				else:
@@ -475,7 +482,7 @@ class watch(object):
 		failed = []
 		last_exc = None
 		log.debug("%s %d watched path%s", my(self), len(self.paths), ses(len(self.paths)))
-		for path in self.paths.keys():
+		for path in list(self.paths):
 			if path not in self.paths_open:
 				try:
 					if not self._add_file(path, **params):
@@ -615,7 +622,7 @@ class watch(object):
 			for path in self._poll_pending:
 				self.last_changes[path] = self._poll_pending[path]
 			self._poll_pending = {}
-			for fd in self._poll_stat.keys():
+			for fd in list(self._poll_stat):
 				path = self.fds_open[fd]
 				fstate = self._poll_get_stat(fd, path)
 				if fstate is None:
@@ -625,7 +632,7 @@ class watch(object):
 					self.last_changes[path] = now
 		else:
 			raise Exception("Unsupported polling mode " + self.get_mode_name())
-		paths = self.last_changes.keys()
+		paths = list(self.last_changes)
 		paths.sort()
 		log.debug("%s Change was to %d path%s", my(self), len(paths), ses(len(paths)))
 		return paths
@@ -726,89 +733,8 @@ class watch(object):
 				return
 		if self._mode == WF_POLLING:
 			log.debug("%s Checking %d open path%s", my(self), len(self._poll_stat), ses(len(self._poll_stat)))
-			for fd in self._poll_stat.keys():
+			for fd in list(self._poll_stat):
 				fstate = self._poll_get_stat(fd, self.fds_open[fd])
 				if fstate is None or self._poll_stat[fd] != fstate:
 					self._poll_trigger()
 					break
-
-
-if __name__ == '__main__':
-	import argparse, random, signal
-
-	def find_open_fds():
-		cnt = 0
-		for fd in range(1024):
-			try:
-				os.fstat(fd)
-				cnt += 1
-			except:
-				pass
-		return cnt
-
-	def catch(sig, frame):
-		global stopping
-		if sig == signal.SIGINT:
-			sys.stderr.write('\nInterrupt!\n')
-		stopping = True
-
-	p = argparse.ArgumentParser(
-		formatter_class=argparse.RawDescriptionHelpFormatter,
-		description="Test the %s module\n%s" %
-		(os.path.splitext(os.path.basename(__file__))[0], watch.__doc__))
-
-	p.add_argument('-v', '--verbose', action='store_true', dest='verbose', help='Verbose logging for debugging')
-	p.add_argument('-q', '--quiet', action='store_true', dest='quiet', help='Warnings and errors only')
-	p.add_argument('-t', '--timeout', action='store', dest='timeout', type=float, default=0.0, help='Aggregation timeout')
-	p.add_argument('-l', '--limit', action='store', dest='limit', type=int, default=0, help='Aggregation limit')
-	p.add_argument('-p', '--polling', action='store_true', dest='polling', help='Using polling instead of file-system events')
-	p.add_argument('-m', '--missing', action='store_true', dest='missing', help='Allow files to be missing')
-	p.add_argument('-r', '--removal', action='store_true', dest='removal', help='Test path removal')
-	p.add_argument('file', nargs='+', help='List of files to watch')
-
-	args = p.parse_args()
-
-	log = logging.getLogger()
-	log.addHandler(logging.StreamHandler())
-	if args.verbose:
-		log.setLevel(logging.DEBUG)
-	if args.quiet:
-		log.setLevel(logging.WARNING)
-
-	if signal.signal(signal.SIGINT, catch) == signal.SIG_IGN:
-		signal.signal(signal.SIGINT, signal.SIG_IGN)
-	signal.signal(signal.SIGHUP, catch)
-	signal.signal(signal.SIGTERM, catch)
-
-	print "%s files open before watch started" % (find_open_fds(),)
-
-	snoop = watch(polling=args.polling, log=log, timeout=args.timeout, limit=args.limit)
-	log.info("Watching in %s mode", snoop.get_mode_name())
-	snoop.add(args.file, missing=True)
-
-	print "%s files open watching %d paths with watch started, before remove" % (find_open_fds(), len(snoop.paths_open))
-
-	#  test removal if that will leave at least one
-	if args.removal and len(args.file) > 1:
-		zapping = args.file[random.randrange(len(args.file))]
-		snoop.remove(zapping)
-		print "Removal test removed", zapping
-
-	print "%s files open watching %d paths with watch started" % (find_open_fds(), len(snoop.paths_open))
-	stopping = False
-	while not stopping:
-		try:
-			ready = select.select([snoop], [], [], 3)
-		except Exception as e:
-			if e[0] != errno.EINTR:
-				log.error("Select failed -- %s", str(e))
-			break
-		if ready == ([],[],[]):
-			snoop.scan()
-			continue
-		print 'Changes detected ...'
-		for path in snoop.get():
-			print '    ', path
-	del snoop
-	print "%s files open with watch removed" % (find_open_fds(),)
-	raise SystemExit(0)
